from random import *
import globs
from image import im
from npc import *
from hero import Hero

class Cell:
	
	def __init__(self,state,type=None):
		self.state = state
		self.wall = False
		self.hole = False
		self.exit = False
		self.visitor = None
		self.contents = None

		self.img = im["ground%i"% randint(0,3)]
		self.lit = False
		
		if type:
			if type == "wall":
				self.wall = True
				self.img = im["wall%i"%randint(0,1)]
			return
		else:
			#if no type
			if not randint(0,10):
				self.wall = True
				self.img = im["wall%i"%randint(0,1)]
			elif not randint(0,30):
				self.hole = True
				self.img = im["hole"]
			elif not randint(0,60):
				self.contents = im["battery"]
			elif not randint(0,100):
				self.contents = im["mine"]
			elif not randint(0,200):
				self.contents = im["medicalHerbs"]
	
	def isFree(self):
		return not self.wall and not self.hole and not self.visitor and not self.contents
	
	def bump(self,who):
		if self.visitor:
			self.visitor.bumpBy(who)
		elif self.hole and who == self.state.hero:
			self.state.msg(["You decide against jumping in the bottomless pit."])
		elif self.wall and who == self.state.hero:
			if not who.lightOn:
				self.state.msg(["You bump into a wall."])
			else:
				for i in range(9):
					self.state.npcTick()			
				if randint(0,7) and who.battery >= 10:
					self.state.msg(["You dig a tunnel into the rock."])
					self.wall = False
					self.img = im["dugout"]
					who.battery -= 9
				elif who.battery < 10:
					self.img = im["damagedWall"]
					who.battery = 0
					who.lightOn = False
					self.state.msg(["You start digging but you light goes out before you finish."])
				else:
					self.state.msg(["You hit the wall with your pickaxe."])
					self.img = im["damagedWall"]
				#add noise that attracts guards
				
			
	
	def setExit(self):
		self.wall = False
		self.hole = False
		self.contents = None
		self.exit = True
		self.img = im["stairs"]
	
	def setHole(self):
		self.wall = False
		self.contents = None
		self.hole = True
		self.img = im["hole"]
	
	def visit(self,who):
		self.visitor = who
		if self.contents == im["mine"]:
			self.contents = None
			self.state.msg(["BOOM!!!"],(255,255,0))
			who.hurt(randint(10,15))
		if self.contents == im["medicalHerbs"]:
			if who.health < who.maxHealth:
				self.contents = None
				who.health += randint(5,7)
				if who == self.state.hero:
					self.state.msg(["You found some medicinal herbs."],globs.itemColor)
				if who.health > who.maxHealth:
					who.health = who.maxHealth
		
		if who != self.state.hero:
			return
		if self.hole == True:
			self.state.msg(["You fell into a bottomless pit... Actually not quite bottomless,"," you realise as you hit the ground at about 55 m/s."],(255,0,0))
			who.img = im["hole"]
			self.state.die()
		if self.exit:
			self.state.nextLevel()
			self.state.msg(["You go up to the next floor."])
		if self.contents == im["battery"]:
			self.contents = None
			self.state.msg(["You found a battery."],globs.itemColor)
			who.battery += randint(20,40)
		if self.contents in [im["gem%i"%i] for i in range(4)]:
			who.gems.append(self.contents)
			self.contents = None
			self.state.msg(["You found a %s gem."%choice(globs.gemAdjs)],globs.itemColor)
		if self.contents == im["whip"]:
			if who.strength < 4:
				self.contents = None
				who.strength = 4
				who.weapon = "a whip"
				self.state.msg(["You found a leather whip."],globs.itemColor)
		if self.contents == im["knife"] and who.strength < 6:
			self.contents = None
			who.strength = 6
			who.weapon = "a knife"
			self.state.msg(["You found a steel knife."],globs.itemColor)





class Map:

	def __init__(self,state):
		self.state = state
		self.ignoreLight = False
		self.cells = []
	
	def buildMap(self,level):
		self.cells = []
		for x in range(globs.WIDTH):
			self.cells.append([])
			for y in range(globs.HEIGHT):
				self.cells[x].append(0)
		import cavern
		cavern.cavernGen(self.state,self.cells)
		
		#punch holes
		for h in range((9+level)*20):
			x,y = self.getFreeCell()
			self.cells[x][y].setHole()
		
		#place player
		x,y = self.getFreeCell()
		if not hasattr(self.state,"hero"):
			self.state.hero = Hero(self.state)
		self.state.hero.x = x
		self.state.hero.y = y
		self.cells[x][y].visitor = self.state.hero
		if level > -8:
			self.cells[x][y].img = im["downstairs"]
		#add exit:
		x,y = self.getFreeCell()
		self.cells[x][y].setExit()
		self.exitX,self.exitY = x,y
		
		#add slaves, less as you go up
		for s in range(-level):
			x,y = self.getFreeCell()
			self.state.npcs.append(Slave(self.state,x,y))
		#add foremen, 1 per 3 slaves + 2
		for f in range(-level/3+2):
			x,y = self.getFreeCell()
			fman = Foreman(self.state,x,y)
			if level != -8:
				fman.hostile = True
			self.state.npcs.append(fman)	
		#guards, more as we go up
		for g in range(level+9):
			x,y = self.getFreeCell()
			self.state.npcs.append(Guard(self.state,x,y))
		#dogs starting on level -7
		for d in range((level+8)*2):
			x,y = self.getFreeCell()
			self.state.npcs.append(Dog(self.state,x,y))
		
		#batteries, less as we go up
		for b in range(-level*2+5):
			x,y = self.getFreeCell()
			self.cells[x][y].contents = im["battery"]
		#medicine
		for m in range(16-(8+level)):
			x,y = self.getFreeCell()
			self.cells[x][y].contents = im["medicalHerbs"]
		#mines, more as we climb
		for m in range(-level*5):
			x,y = self.getFreeCell()
			self.cells[x][y].contents = im["mine"]
		#gems
		for g in range(5):
			x,y = self.getFreeCell()
			self.cells[x][y].contents = im["gem%i"%randint(0,3)]

	def getFreeCell(self):
		x,y = randint(0,globs.WIDTH-1),randint(0,globs.HEIGHT-1)
		while not self.cells[x][y].isFree():
			x,y = randint(0,globs.WIDTH-1),randint(0,globs.HEIGHT-1)
		return x,y
	
	def isFree(self,x,y):
		seenHole = self.cells[x][y].hole and self.cells[x][y].lit
		return not self.cells[x][y].wall and not self.cells[x][y].visitor and not seenHole
	
	def npcFree(self,x,y):
		mine = self.cells[x][y].contents == im["mine"] and randint(0,20)
		return self.isFree(x,y) and not self.cells[x][y].hole and not mine
	
	def doLight(self,sources):
		for x in self.cells:
			for c in x:
				c.lit = False
		for s in sources:
			for x in range(s[0]-2,s[0]+3):
				for y in range(s[1]-2,s[1]+3):
					if 0 <= x < globs.WIDTH and 0 <= y < globs.HEIGHT:
						self.cells[x][y].lit = True

	
	def render(self,screen):
		for x,line in enumerate(self.cells):
			for y,cell in enumerate(line):
				if self.ignoreLight or cell.lit:
					screen.blit(cell.img,(x*globs.CELLW,y*globs.CELLH-4))
					if cell.contents:
						screen.blit(cell.contents,(x*globs.CELLW,y*globs.CELLH-4))