from random import *
import globs
from image import im
from person import Person

class NPC(Person):

	def __init__(self,state,x,y):
		Person.__init__(self,state,x,y)
	
	def tick(self):
		pass
	
	def homeOnHero(self):
		dx,dy = 0,0
		hero = self.state.hero
		if self.x < hero.x:
			dx = 1
		elif self.x > hero.x:
			dx = -1
		if self.y < hero.y:
			dy = 1
		elif self.y > hero.y:
			dy = -1
		self.move((dx,dy))
	
	def seesHero(self):
		for dx in range(-2,3):
			for dy in range(-2,3):
				x,y = self.x + dx , self.y + dy
				if 0 <= x < globs.WIDTH and 0 <= y < globs.HEIGHT:
					if self.state.map.cells[x][y].visitor == self.state.hero:
						return True
		return False
	
	def wander(self):
		dir = (randint(-1,1),randint(-1,1))
		if not randint(0,1):
			self.move(dir)

class Dog(NPC):
	
	def __init__(self,state,x,y):
		NPC.__init__(self,state,x,y)
		self.lightOn = False
		self.name = "the %s guard dog" % choice(globs.animalAdjs)
		self.img = im["dog"]
		self.strength = 1
		self.health = self.maxHealth = 7
	
	def tick(self):
		if self.seesHero() or not randint(0,3):
			self.homeOnHero()
		else:
			self.wander()

class Guard(NPC):

	def __init__(self,state,x,y):
		NPC.__init__(self,state,x,y)
		self.name = "the %s guard" % choice(globs.humanAdjs)
		self.img = im["guard"]
		self.lastSeen = None
	
	def tick(self):
		if self.seesHero():
			self.lastSeen = (self.state.hero.x,self.state.hero.y)
			if randint(0,2):
				self.state.msg(["%s shot you."% self.name.capitalize()],globs.combatColor)
				self.state.hero.hurt(randint(3,5))
			else:
				self.homeOnHero()
		else:
			if self.lastSeen:
				self.move((cmp(self.lastSeen[0],self.x),cmp(self.lastSeen[1],self.y)))
				if (self.x,self.y) == self.lastSeen:
					self.lastSeen = None
			else:
				self.wander()
	
	def die(self):
		if not randint(0,3):
			if not self.state.map.cells[self.x][self.y].contents:
				self.state.map.cells[self.x][self.y].contents = im["knife"]
		Person.die(self)

class Foreman(NPC):
	def __init__(self,state,x,y):
		NPC.__init__(self,state,x,y)
		self.name = "the %s foreman" % choice(globs.humanAdjs)
		self.img = im["foreman"]
		self.hostile = False
		self.dexterity = 4
		self.health = 10
	
	def tick(self):
		if self.seesHero():
			if not self.hostile:
				if not randint(0,10):
					self.state.msg(["%s says: Hey you, get back to work."%self.name.capitalize()],globs.dialogueColor)
			else:
				self.homeOnHero()
		else:
			self.wander()
	
	def bumpBy(self,other):
		if other == self.state.hero:
			self.hitBy(other)
			self.hostile = True
	
	def die(self):
		if not randint(0,5):
			if not self.state.map.cells[self.x][self.y].contents:
				self.state.map.cells[self.x][self.y].contents = im["whip"]
		Person.die(self)

class Slave(NPC): #todo add dialogue that tells that real way to exit
	
	def __init__(self,state,x,y):
		NPC.__init__(self,state,x,y)
		self.name = "the %s slave" % choice(globs.humanAdjs)
		self.img = im["slave"]
		self.exitDir = "meh"
		self.dexterity = 0
		self.health = self.maxHealth = randint(2,5)
		self.dialogue = ["We'd better get back to work.",
				"Someday I'm going to rebel.",
				"Watch out for the mines.",
				"Why did they put mines in these mines?",
				"<dummy last one for exit direction>"
				]
	
	def tick(self):
		exitVDir = exitHDir = ""
		if self.x - self.state.map.exitX > 4:
			exitHDir = "west"
		elif self.state.map.exitX - self.x > 4:
			exitHDir = "east"
		if self.y - self.state.map.exitY > 4:
			exitVDir = "north"
		elif self.state.map.exitY - self.y > 4:
			exitVDir = "south"
		self.exitDir = "to the %s%s"%(exitVDir,exitHDir)
		if exitVDir == exitHDir == "":
			self.exitDir = "nearby"
		self.dialogue[-1] = "The exit is %s." % self.exitDir
		self.wander()
	
	def bumpBy(self,other):
		if other == self.state.hero:
			self.state.msg(["%s says: %s"%(self.name.capitalize(),choice(self.dialogue))],globs.dialogueColor)
		elif "foreman" in other.name:
			self.state.msg(["%s whipped %s."%(other.name.capitalize(),self.name)])
			self.hurt(1)
	
	def die(self):
		if not randint(0,5):
			if not self.state.map.cells[self.x][self.y].contents:
				self.state.map.cells[self.x][self.y].contents = im["gem%i"%randint(0,globs.numGems-1)]
		Person.die(self)